<?php

namespace AustraliaPost\Extensions\Business\Endpoints;


use AustraliaPost\Extensions\Business\API\Business;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Feature;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item;
use AustraliaPost\Extensions\Business\API\Wrapper\Price_Item_Clearance_Description;
use AustraliaPost\Extensions\Business\API\Wrapper\Shipment_Request;
use AustraliaPost\Extensions\Business\Helpers\Utilities;
use WC_Order;

class Create_Shipments_Endpoint extends Abstract_Endpoint {

	const LABELS_SHIPMENTS_META_KEY = '_labels_pro_created_shipments';
	const LABELS_SHIPMENTS_DATE_META_KEY = '_labels_pro_shipment_creation_datetime';

	public function callback( $data ) {
		$order = $this->get_order($data);

		$shipments =  $data['shipments'];
		return $this->create_shipment($order, $shipments, $data['has_dangerous_goods']);
	}

	/**
	 * @param WC_Order $order
	 * @param array $shipments
	 * @param bool $is_dangerous
	 * @return array
	 */
	private function create_shipment($order, $shipments, $is_dangerous)
	{
		$customerReference = sprintf('Order %s', $order->get_order_number());
		$customerReference = apply_filters('labels_pro_shipment_customer_reference', $customerReference, $order->get_id() );
		$shipmentRequest = (new Shipment_Request())
			->setFrom($this->get_from_address())
			->setTo($this->get_to_address($order))
			->setReference($order->get_order_number())
			->setCustomerReference1(substr($customerReference, 0, 50));

		foreach ($shipments as $index => $shipment) {
			$cubicWeight = $this->cubicWeight($shipment);
			if ($shipment['weight'] > 1000 && $shipment['product_id'] === 'B20' && max($cubicWeight, $shipment['weight']) > 5000) {
				$shipment['product_id'] = 'B21';
				$shipments[$index]['product_id'] = 'B21';
			}

			if ($shipment['weight'] > 1000 && $shipment['product_id'] === 'B30' && max($cubicWeight, $shipment['weight']) > 5000) {
				$shipment['product_id'] = 'B31';
				$shipments[$index]['product_id'] = 'B31';
			}

			$priceItem = (new Price_Item(Utilities::accountType()))
				->setLength($shipment['length'] / 10)
				->setWidth($shipment['width'] / 10)
				->setHeight($shipment['height'] / 10)
				->setWeight($shipment['weight'] / 1000)
				->setProductId($shipment['product_id'])
				->setItemReference($order->get_id() .'#' . $index);

			if ($is_dangerous === true) {
				$priceItem->setContainsDangerousGoods(true);
			}

			$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment);

			if ($shipment['extra_cover'] > 0) {
				$extra_cover_feature = new Price_Feature();
				$extra_cover_feature->setCode('TRANSIT_COVER');
				$extra_cover_feature->setAttributes(['cover_amount' => round(floatval($shipment['extra_cover']), 2)]);
				$priceItem->addFeature($extra_cover_feature);
				if ($shipment['extra_cover'] > 500 && $shipment['signature_on_delivery'] === false) {
					$priceItem = $this->addSignatureOnDelivery($priceItem, $shipment, true);
				}
			}

			$priceItem = $this->addClearanceItems($priceItem, $shipment);

			$shipmentRequest->addItem( $priceItem );
		}

		$shipmentsResponse = (new Business())->create_shipment($shipmentRequest);

		if (isset($shipmentsResponse['errors'])) {
			$this->abort($shipmentsResponse['errors']);
		}

		$this->store_in_order($shipmentsResponse, $order);

		do_action('labels_pro_shipment_for_order_created', $order->get_id());

		return $this->filter_for_app($shipmentsResponse);

	}

	public function action() {
		return 'create_shipments';
	}

	/**
	 * @param array $shipmentsResponse
	 * @param WC_Order $order
	 */
	private function store_in_order( $shipmentsResponse, $order )
	{

		if (!isset($shipmentsResponse['shipments'])) {
			return;
		}

		$order->update_meta_data(self::LABELS_SHIPMENTS_META_KEY, Utilities::_json_encode( $shipmentsResponse['shipments']));
		$order->update_meta_data(self::LABELS_SHIPMENTS_DATE_META_KEY, date('Y-m-d H:i:s'));
		$order->save();
	}

	/**
	 * @param array $shipmentsResponse
	 *
	 * @return array
	 */
	private function filter_for_app( $shipmentsResponse )
	{
		$response = [];

		if (!isset($shipmentsResponse['shipments'])) {
			return $response;
		}

		foreach ( $shipmentsResponse['shipments'] as $key => $shipment ) {
			if (!isset($shipment['items'])) {
				continue;
			}
			$item = $shipment['items'][0];
			$response[$key]['shipment_id'] = (isset($shipment['shipment_id']))? $shipment['shipment_id']: null;
			$response[$key]['item_id'] = $item['item_id'];
			$response[$key]['article_id'] = $item['tracking_details']['article_id'];
			$response[$key]['consignment_id'] = $item['tracking_details']['consignment_id'];
		}

		return $response;
	}

	/**
	 * @param Price_Item $priceItem
	 * @param array $item
	 *
	 * @return Price_Item
	 */
	private function addClearanceItems($priceItem,  $item ) {
		if (!isset($item['clearance_items']) && is_array($item['clearance_items'])) {
			return $priceItem;
		}

		foreach ($item['clearance_items'] as $clearanceItem) {
			$itemContent = new Price_Item_Clearance_Description();
			$itemContent->setDescription($clearanceItem['description'])
			            ->setQuantity($clearanceItem['quantity'])
			            ->setValue($clearanceItem['value'])
			            ->setCountryOfOrigin($clearanceItem['country_of_origin']);
			$priceItem->addItemContent($itemContent);
		}

		return $priceItem;
	}
}
